***************************************************************************
*                                                                         *
*                         MC68HC705V8 MDLC Module                         *
*                         Example Driver Software                         *
*                                                                         *
*     This software is an example of a basic software routine which       *
* can be used with the Message Data Link Controller (MDLC) module on the  *
* Motorola MC68HC705V8 microcontroller. This simple example program loads *
* a message into MCU RAM and then calls the transmit subroutine TRANSMIT  *
* which loads the message into the MDLC module transmit buffer and then   *
* initiates the message transmission. Once the message transmission is    *
* initiated, the Real Time Interrupt (RTI) function on the MCU's Core     *
* Timer is activated. If the transmission successful interrupt does not   *
* occur before 8 RTI interrupts have occurred (about 30ms), the trans-    *
* mission is aborted. When the transmission successful interrupt does     *
* occur, the RTI interrupt is disabled. While waiting for the message to  *
* complete successfully or to be aborted, the program simply waits in a   *
* time delay loop.                                                        *
*     This basic routine interprets received messages as either physically*
* addressed or functionally addressed. In J1850, a functionally addressed *
* message is indicated if Bit 2 of the Pri/type byte (1st byte) of the    *
* message is clear, while for physically addressed messages, Bit 2 will   *
* be set. No other message type determination is made, and no in-frame    *
* response byte is (or can be) returned.                                  *
*     When a message is received, the MDLC interrupt service routine will *
* compare the target address of the received message with 4 pre-defined   *
* message identifiers. If the target address of the message matches one   *
* of the four, the message is retrieved and saved in a buffer reserved for*
* that message ID. If the target address of the received message does not *
* match, the message is discarded. Also, if the received message is a     *
* physically addressed message, the target address of the received        *
* message is compared with the assigned node address, and if it matches   *
* the message is then retrieved and stored in a RAM buffer reserved for   *
* physically addressed messages, along with the source address of the     *
* message originator.                                                     *
*     Though simple, this software demonstrates all of the basic features *
* of the MDLC module, and how they can be used for performing communi-    *
* cation across a J1850-compatible multiplex bus.                         *
*                                                                         *
*                           Revision History                              *
*                                                                         *
*      Rev 0.1 (initial release)         Chuck Powers         5/03/93     *
*      Rev 0.2 Updated transfer routines                                  *
*              between MCU RAM and the                                    *
*              MDLC transmit and receive                                  *
*              buffers.                                       7/08/93     *
*      Rev 0.3 Added initialization of                                    *
*              MISC register                                  8/05/93     *
*                                                                         *
***************************************************************************

***************************************************************************
*****                            Equates                              *****
***************************************************************************

*** MC68HC705V8 Register Equates ***

porta	equ	$00	;Port A Data Register
portb	equ	$01	;Port B Data Register
portc	equ	$02	;Port C Data Register
portd	equ	$03	;Port D Data Register

ddra	equ	$04	;Port A Data Direction Register
ddrb	equ	$05	;Port B Data Direction Register
ddrc	equ	$06	;Port C Data Direction Register

ctcsr	equ	$08	;Core Timer Control/Status Register
ctcount	equ	$09	;Core Timer Count Register

mcr  	equ	$0e	;MDLC Control Register
msr  	equ	$0f	;MDLC Status Register
mtcr 	equ	$10	;MDLC Transmit Control Register
mrsr 	equ	$11	;MDLC Receive Status Register
txdata0	equ	$20	;Beginning of MDLC Tx Buffer
rxdata0	equ	$34	;Beginning of MDLC Rx Buffer(s)
rxdata1	equ	$35	;Target Address storage in MDLC Rx Buffer(s)
rxdata2	equ	$36	;Source Address storage in MDLC Rx Buffer(s)
rxdata3	equ	$37	;Beginning of MDLC Rx Data Bytes

misc	equ	$2f	;Miscellaneous Register

*** MCR Bit Assignments ***

rxbm	equ	7	;Receive Block Mode Enable     
txab	equ	6	;Transmit Abort                 
r1  	equ	5	;\ J1850 Bus Communication      
r0  	equ	4	;/ Rate Select Bits             
ie  	equ	1	;MDLC Interrupt Enable          
wcm 	equ	0	;MDLC Wait Clock Mode Select    

*** MSR Bit Assignments ***

txms	equ	3	;Transmitter Successful Indicator  
rxms	equ	2	;Receiver Successful Indicator

*** MISC Bit Assignments ***

igns	equ	7	;Ignition Status  
oce	equ	6	;Output Compare Enable  
pdc	equ	0	;Power Down Control Indicator

*** CTCSR Bit Assignments ***

ctof	equ	7	;Core Timer Overflow Flag      
rtif	equ	6	;Real Time Interrupt Flag       
tofe	equ	5	;Timer Overflow Interrupt Enable
rtie	equ	4	;Real Time Interrupt Enable     
tofc	equ	3	;Timer Overflow Flag Clear Bit
rtfc	equ	2	;RTI Interrupt Flag Clear Bit   
rt1	equ	1	;\ Real Time Interrupt          
rt0 	equ	0	;/ Rate Select Bits             

*** Receive Control Bit Assignments

type	equ	2	;Rxdata0, Bit 2 (Received Msg Type Indicator)

*** General Equates ***

ram	equ	$0040	;Beginning of user RAM
rom	equ	$0d00	;Beginning of user ROM
mdlc	equ	$1300	;Beginning of MDLC Interrupt Service Routine
ctimer	equ	$1000	;Beginning of Core Timer Int. Service Routine
sub 	equ	$2000	;Beginning of MDLC Subroutines      
vectors	equ	$3ff2	;Beginning of user vectors
none	equ	$0000	;Bogus Location
mor	equ	$3c00	;Mask Option Register Location
copr	equ	$3ff0	;COP Watchdog Reset Location

*** Node Address ***

node	equ	$77	;MC68HC705V8 Node physical address

*** Functional Message I.D.'s ***

id1	equ	$1a
id2	equ	$20
id3	equ	$5e
id4	equ	$d3

***************************************************************************
*****                   HC05 RAM Storage Assignments                  *****
***************************************************************************

	org	ram

*** Variable Data Byte Storage ***

data_byt	rmb	$1	;Tx Message Variable Data Byte Storage

*** Core Timer Registers ***

rticnt	rmb	$1	;RTI Interrupt Count Register

*** Transmit Message Buffer ***

txcount	rmb	$1	;Host Transmit Message Byte Count
txbuf	rmb	$1	;Start of Host Transmit Message Buffer
target	rmb	$1	;Tx Message Target Address
source	rmb	$1	;Tx Message Source Address
data1	rmb	$1	;Tx Message 1st Data Byte
data2	rmb	$1	;Tx Message 2nd Data Byte
data3	rmb	$1	;Tx Message 3rd Data Byte
data4	rmb	$1	;Tx Message 4th Data Byte
data5	rmb	$1	;Tx Message 5th Data Byte
data6	rmb	$1	;Tx Message 6th Data Byte
data7	rmb	$1	;Tx Message 7th Data Byte
data8	rmb	$1	;Tx Message 8th Data Byte

*** Received Message Storage ***

buff_ndx	rmb	$1	;Temporary storage for receive message index
byte_cnt	rmb	$1	;Temporary storage for received message count

ncount	rmb	$1	;RAM holding # of Data Bytes in last N-N msg
nsource	rmb	$1	;RAM holding source address of last N-N msg
nbuff	rmb	$8	;RAM holding last received N-N message

fmsg1	rmb	$1	;Pointer to RAM holding funct. message w/id1
fmsg2	rmb	$1	;Pointer to RAM holding funct. message w/id2
fmsg3	rmb	$1	;Pointer to RAM holding funct. message w/id3
fmsg4	rmb	$1	;Pointer to RAM holding funct. message w/id4

buff1	rmb	$8	;RAM holding last received message w/id1
buff2	rmb	$8	;Ram holding last received message w/id2
buff3	rmb	$8	;RAM holding last received message w/id3
buff4	rmb	$8	;RAM holding last received message w/id4

***************************************************************************
*                            Main Program                                 *
***************************************************************************

*************************************************************************
* This is the main line program for this example. Following a reset,    *
* this routine will reset the MC68HC705V8, preparing it for J1850       *
* communication, and then transmit a message made up of: Pri/type = $31 *
* Target Address = $A1, Source Address = $77, and a variable data byte. *
* Following the initiation of transmission, the variable data byte is   *
* incremented, and then a delay loop is entered which allows the trans- *
* mission to take place. Periodically during the delay, the COP timer is*
* cleared. Once the delay loop is complete, the Tx routine begins again.*
*************************************************************************
	org	rom

	jsr	v8_rst		;Call subroutine to reset the MCU

	clr	data_byt

doover:	sei		;Set I-bit to prevent unwanted interrupts

	lda	#$31	;Load Pri/Type byte ($31) into
	sta	txbuf	;1st byte of RAM Tx buffer

	lda	#$a1	;Load target address byte into
	sta	target	;second byte of RAM Tx buffer

	lda	data_byt	;Load variable data byte into
	sta	data1	;first data byte location of RAM Tx buffer

	lda	#$04	;Load number of bytes to be transmitted
	sta	txcount	;into RAM location TXCOUNT

	jsr	transmit	;Call subroutine to transmit message
			;onto multiplex bus

	cli		;Clear I-bit to allow interrupts

	inc	data_byt	;Increment transmitted variable data byte

	ldx	#$3f	;Begin delay loop...

lp1:	lda	#$ff
lpo:	deca
	bne	lpo

*** Kick Watchdog ***

	lda	#$00		;Load $00 into COPR to kick
	sta	copr		;the COP watchdog

	decx
	bne	lp1	;end delay loop
		
skip:	jmp	doover

***************************************************************************
*****                   MCU Initialization Routine                    *****
***************************************************************************

*************************************************************************
* This subroutine initializes all of the MCU modules and RAM locations  *
* used in this example routine. This includes the MISC register, the    *
* MDLC module, the Core Timer, the node address, and the RTI counter.   *
* The COP Watchdog is then kicked for good luck!                        *
*************************************************************************

	org	sub

v8_rst:	sei			;To reset the MCU, JSR to here

*** Initialize Miscellaneous for Voltage Regulator Operation ***

	lda	#%00000001	;Output Compare disabled,
	sta	misc		;Primary voltage regulator enabled

*** Initialize MDLC for J1850 Comm. ***

	lda	#%00101110	;B7-rxbm, B6-txab, B5:4-Rate Select
	sta	mcr		;B1-Interrupt Enable, B0-Wait Clock Mode
				;Also, MTST1:0=1 for EVS

*** Initialize Core Timer ***

	lda	#%00000010	;Initialize the Core Timer for using
	sta	ctcsr		;the RTI for timing transmissions

*** Initialize 05V8 Node Address ***

	lda	#node		;Store the node address into
	sta	source		;the correct location in the RAM
				;Transmit buffer
*** Clear RTI Counter ***

	clr	rticnt	;Clear RTI counter, just to make sure
			;it begins life=0

*** Initialization of Receive Message Buffer Pointers ***

	lda	#buff1	;Load location of message buffer w/id1
	sta	fmsg1	;in message buffer pointer fmsg1

	lda	#buff2	;Load location of message buffer w/id2
	sta	fmsg2	;in message buffer pointer fmsg2

	lda	#buff3	;Load location of message buffer w/id3
	sta	fmsg3	;in message buffer pointer fmsg3

	lda	#buff4	;Load location of message buffer w/id4
	sta	fmsg4	;in message buffer pointer fmsg4

*** Kick Watchdog ***

	lda	#$00		;Load $00 into COPR to kick
	sta	copr		;the COP watchdog

	cli			;Clear I-bit, enabling interrupts

	rts			;Return from Reset subroutine

***************************************************************************
*****                          Subroutines                            *****
***************************************************************************

*** MDLC Transmit Subroutine ***

**********************************************************************
* This subroutine transfers the data in the RAM transmit buffer into *
* the MDLC Tx buffer, loads the number of bytes to be transmitted    *
* into the MTCR, initiating the message transmission, and then       *
* enables the RTI interrupt to count RTI's until either the message  *
* is transmitted successfully, or 30ms has passed. If 30ms passes    *
* without the message transmitting successfully, the message will    *
* be aborted automatically by the software.                          *
**********************************************************************

transmit:	ldx	txcount	;Load the number of bytes to be transmitted
			;into the X-register

nexttx:	decx		;Now decrement X to get it to the correct
			;value to begin an indexed transfer of the
			;data from RAM into the MDLC Tx buffer

	lda	txbuf,x	;Load message data byte (last byte first)
	sta	txdata0,x	;into the MDLC Tx register

	tstx		;Test the X-Register to see if it is 0

	bne	nexttx	;If X<>0, indicating the last byte has not
			;been transferred, then go get the next byte

	lda	txcount	;Otherwise, load Tx byte count into Acc.

	sta	mtcr	;Then store in MTCR, initiating transmission

	bset	rtfc,ctcsr		;Clear RTI interrupt flag, in case
				;it has been set

	bset	rtie,ctcsr	;Enable RTI interrupt to track how
				;long the transmission is pending

	rts			;Then return from subroutine

***************************************************************************
*****                  MDLC Interrupt Routine Service                 *****
***************************************************************************

**********************************************************************
* The MDLC interrupt service routine handles both the transmit and   *
* receive interrupts of the CPU generated by the MDLC module. When a *
* receive interrupt is serviced, the software automatically checks   *
* the message type of the received message. If it is a physically    *
* addressed message, the target address of the message is compared   *
* to the node address, and if it matches, the message data bytes and *
* source address are retrieved and stored in RAM. If it is a funct-  *
* ionally address message, the target address is compared to 4 pre-  *
* viously defined target addresses and if there is a match, the data *
* bytes of the message are stored in that message ID's corresponding *
* RAM buffer. The source or target address are not retained, since   *
* it is assumed that the receiver must already know what the message *
* pertains to.                                                       *
*     When a transmit interrupt occurs, the MDLC module will         *
* acknowledge the successful transmission by clearing the transmit   *
* interrupt, and then disabling the RTI interrupt and clearing the   *
* RTI counter register. The application is then ready for another    *
* transmission.                                                      *
**********************************************************************

	org	mdlc

	brset	rxms,msr,rxint		;If an receive interrupt, go to
					;receive interrupt service routine

	jmp	txint		;Otherwise, go to transmit interrupt
				;service routine

*** Receive Message Successful Interrupt Handling Routine ***

rxint:	brclr	type,rxdata0,funct	;Is Bit 2 of the Pri/Type byte
					;of the message set, indicating
					;a physically addressed message?

	lda	rxdata1	;If physically addressed, 
			;load Target Address into Acc.

	cmp	#node	;Does it match my Node Address?

	bne	rxdone	;If not, ignore message and release Rx buffer

	lda	mrsr	;Otherwise, load received message
			;byte count into Acc.

	sub	#$03	;Subtract $02 for correct # of data bytes

	sta	ncount	;Then store in N-N message byte count storage

	tax		;Also store # of data byte in X-register

nextn:	decx		;Then decrement X, for correct offset
			;for indexed transfer of data from the
			;Rx buffer to the RAM buffer

	lda	rxdata3,x	;Load next data byte (last byte first)
	sta	nbuff,x	;into Node-Node RAM buffer

	tstx		;Test the X-Register to see if it is 0

	bne	nextn	;If X<>0, indicating the last byte has not
			;been transferred, then go get the next byte

	lda	rxdata2	;Load Source Address of receive message
	sta	nsource	;into RAM location NSOURCE

	jmp	rxdone	;Then go release the Rx buffer and finish up

funct:	clrx		;Clear Index Register

	lda	rxdata1	;Compare target address of received
	cmp	#id1	;message with ID#1

	beq	get_msg	;If it matches, go get message

	incx		;Otherwise increment X-register, and then...

	cmp	#id2	;Compare target address with ID#2

	beq	get_msg	;If it matches, go get message

	incx		;Otherwise increment X-register, and then...

	cmp	#id3	;Compare target address with ID#3

	beq	get_msg	;If it matches, go get message

	incx		;Otherwise increment X-register, and then...

	cmp	#id4	;Compare target address with ID#4

	beq	get_msg	;If it matches, go get message

	jmp	rxdone	;If not, go release Rx buffer,
			;discarding the received message

get_msg:	lda	mrsr	;Read received message byte count

	sub	#$03	;Subtract $03 to account for message header

	sta	byte_cnt	;And store in temporary storage

	add	fmsg1,x	;Add starting address of appropriate
			;message buffer to get ending address

	deca		;Decrement to get correct address index

	sta	buff_ndx	;and store in temporary storage

next_byt:	ldx	byte_cnt	;Then load message byte count into X-reg.

	lda	rxdata2,x	;Read received message data byte

	ldx	buff_ndx	;Now load message buffer index into X-reg.

	sta	,x	;and store message into message buffer

	dec	buff_ndx	;Decrement message buffer index

	dec	byte_cnt	;Decrement received byte count

	bne	next_byt	;If not zero, go get next byte

rxdone:	sta	mrsr	;Write data in Acc. into MRSR to clear
			;RXMS interrupt

*** Tx Message Successful Interrupt Handling Routine ***

txint:	brclr	txms,msr,mdlcclr	;Check to see if a Tx interrupt
					;is pending - if not, go return
					;from the interrupt

	clr	rticnt		;Clear RTI Counter register

	bclr	rtie,ctcsr	;Then disable the RTI interrupt

	lda	#$00		;Write $00 into the MTCR
	sta	mtcr		;to clear TXMS interrupt

	brclr	rxms,msr,mdlcclr	;If Rx interrupt is not pending,
					;then finish up interrupt routine

	jmp	rxint		;Otherwise, go check out the
				;pending receive interrupt

mdlcclr:	rti			;Return from interrupt

***************************************************************************
*                  Core Timer Interrupt Service Routine                   *
***************************************************************************

******************************************************************
* This routine handles the Real Time Interrupt (RTI) interrupts  *
* enabled by the routine which initiates MDLC transmissions. The *
* RTI interrupts occur every 3.9ms. Following the initiation of  *
* a transmission by the MDLC module, if the TXMS bit is not set  *
* before 8 RTI interrupts are recorded (approximately 30ms), the *
* transmission will be aborted. This routine records and counts  *
* the number of RTI interrupts, and when the count reaches 8,    *
* the message transmission is aborted automatically by software. *
******************************************************************

	org	ctimer

*** RTI Interrupt Service Routine ***

	bset	rtfc,ctcsr	;Clear RTI interrupt flag

	lda	rticnt		;Read current RTI count

	cmp	#$07		;Have 7 RTI's already occurred?

	bhs	dumptx		;If so, go abort the transmission

	inc	rticnt		;If not, increment RTICNT

	rti			;Then return from interrupt

dumptx:	bset	txab,mcr		;Set the Tx Abort bit in the MCR

doabort:	brset	txab,mcr,doabort	;Wait until Tx Abort bit clears,
					;indicating that the transmission
					;has successfully been halted

	clr	rticnt		;Then clear the RTICNT register

	bclr	rtie,ctcsr	;Also disable RTI interrupt

	rti			;Then return from interrupt

***************************************************************************
*****                 MC68HC705V8 Mask Option Register                *****
***************************************************************************

	org	mor

	fcb	%01111001		;Regulator enabled, set Vdd clamp,
				;MDLC power up enabled, LVR enabled,
				;STOP mode disabled, IRQ edge sensitive,
				;COP enabled

***************************************************************************
*****                    MC68HC705V8 Reset Vectors                    *****
***************************************************************************

	org	vectors

	fdb	ctimer		;Core Timer interrupt vector
	fdb	none			;SPI interrupt vector
	fdb	mdlc			;MDLC interrupt vector
	fdb	none			;16-Bit Timer interrupt vector
	fdb	none			;IRQ interrupt vector
	fdb	none			;SWI interrupt vector
	fdb	rom			;Reset vector

***************************************************************************
*****                        End of the World                         *****
***************************************************************************
